---------------------------------------------------------------------------
--- This library contains functions for sending emails.
--- The implementation might need to be adapted to the local
--- environment.
---
--- @author Michael Hanus
--- @version February 2007
---------------------------------------------------------------------------

module Mail(sendMail,MailOption(..),sendMailWithOptions) where

import List(intersperse)
import IOExts(execCmd)
import IO(hPutStrLn,hClose)

--- Sends an email via mailx command.
sendMail :: String -> String -> String -> String -> IO ()
sendMail from to subject contents = do
  execMailCmd ("mailx -r \""++from++"\" -s \""++subject++"\" "++to) contents

--- Options for sending emails.
--- @cons CC  - recipient of a carbon copy
--- @cons BCC - recipient of a blind carbon copy
--- @cons TO  - recipient of the email
data MailOption = CC String | BCC String | TO String

--- Sends an email via mailx command and various options.
--- Note that multiple options are allowed, e.g., more than one CC option
--- for multiple recipient of carbon copies.
sendMailWithOptions :: String -> String -> [MailOption] -> String -> IO ()
sendMailWithOptions from subject options contents = do
  execMailCmd ("mailx -r \""++from++"\" -s \""++subject++"\" "++
               (if null bccs then "" else "-b \""++bccs++"\" ") ++
               (if null ccs  then "" else "-c \""++ccs++"\" ") ++  tos)
              contents
 where
   tos  = concat (intersperse " " [ s | TO s <- options ])
   ccs  = concat (intersperse " " [ s | CC s <- options ])
   bccs = concat (intersperse " " [ s | BCC s <- options ])

--- Executes a command to send an email and pass the contents via stdin.
--- Note that \r characters in the contents are removed due to problems
--- with such contents in some Unix environments.
execMailCmd cmd contents = do
  (sin,_,_) <- execCmd cmd
  hPutStrLn sin (filter isUnixChar contents)
  hClose sin
 where
  isUnixChar c = c /= '\r'

